{% set minttl = minttl | default(None) %}
{% set maxttl = maxttl | default(None) %}

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	recursion yes;
	dnssec-validation no;
{% if minttl != None %}
	min-delegation-ttl @minttl@;
{% endif %}
{% if maxttl != None %}
	max-delegation-ttl @maxttl@;
{% endif %}
};

zone "." {
	type hint;
	file "root.hint";
};

{% include "_common/controls.conf.j2" %}
