options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	notify yes;
	ixfr-from-differences yes;
	check-integrity no;
	minimal-responses no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "example" {
	type primary;
	file "example.db";
	also-notify { 10.53.0.2; };
};

zone "dynamic" {
	type primary;
	file "dynamic.db";
	allow-update { any; };
	also-notify { 10.53.0.2; };
};
