// NS3

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	dnssec-validation no;
};


{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint.blackhole";
};

dnssec-policy "default-dynamic" {
	inline-signing no;
};

zone "example" {
	type primary;
	allow-update { any; };
	file "example.db";
};

zone "relaxed" {
	type primary;
	allow-update { any; };
	check-svcb no;
	file "relaxed.db";
};

zone "nsec3param.test" {
	type primary;
	allow-update { any; };
	file "nsec3param.test.db.signed";
};

zone "dnskey.test" {
	type primary;
	allow-update { any; };
	file "dnskey.test.db.signed";
};

zone "many.test" {
	type secondary;
	primaries { 10.53.0.1; };
	allow-update-forwarding { any; };
	file "many.test.bk";
};

zone "delegation.test" {
	type primary;
	allow-update { any; };
	file "delegation.test.db.signed";
};

zone "too-big.test" {
	type primary;
	allow-update { any; };
	max-records 3;
	file "too-big.test.db";
};

zone "many-updates.test" {
	type primary;
	allow-update { any; };
	file "many-updates.test.db";
};


/* Zone for testing CDS and CDNSKEY updates from other provider */
zone "multisigner.test" {
	type primary;
	allow-update { any; };
	dnssec-policy "default-dynamic";
	file "multisigner.test.db";
};
