{% set filtertype = filtertype | default("aaaa") %}
{% set family = family | default("v4") %}

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { fd92:7065:b8e:ffff::1; };
	recursion no;
	dnssec-validation no;
	notify yes;
	minimal-responses no;
};

{% if family == "v6" %}
        plugin query "@TOP_BUILDDIR@/filter-@filtertype@.@DYLIB@" {
                filter-@filtertype@-on-v6 yes;
                filter-@filtertype@ { fd92:7065:b8e:ffff::1; };
        };
{% else %}
        acl filterees { 10.53.0.1; };

        plugin query "@TOP_BUILDDIR@/filter-@filtertype@.@DYLIB@" {
                filter-@filtertype@-on-v4 yes;
                filter-@filtertype@ { filterees; };
        };
{% endif %}

{% include "_common/controls.conf.j2" %}


zone "." { type primary; file "root.db"; };
zone "signed" { type primary; file "signed.db.signed"; };
zone "unsigned" { type primary; file "unsigned.db"; };
