// NS4

options {
	query-source address 10.53.0.4;
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	recursion yes;
	minimal-responses no;

	nta-lifetime 12s;
	nta-recheck 9s;
	validate-except { corp; };

	dnssec-validation yes;
};

include "trusted.conf";

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "corp" {
	type static-stub;
	server-addresses { 10.53.0.2; };
};
