// NS4

options {
	query-source address 10.53.0.4;
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
	minimal-any no;
};

zone "." {
	type primary;
	file "root.db";
};

zone "moves" {
	type primary;
	file "moves.db";
};

zone "child.server" {
	type primary;
	file "child.server.db";
};

zone "tld" {
	type primary;
	file "tld.db";
};

zone "broken" {
	type primary;
	file "broken.db";
};

zone "sourcens" {
    type primary;
    file "sourcens.db";
};

zone "v4only.net" {
	type primary;
	file "v4only.net.db";
};

{% include "_common/controls.conf.j2" %}
