{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
};

zone "example" {
	type primary;
	file "example.db.signed";
};

zone "sub.example" {
	type primary;
	file "sub.example.db.signed";
};

zone "initially-unavailable" {
	type primary;
	file "initially-unavailable.db.signed";
	allow-transfer { 10.53.0.254; };
};

zone "verify-addzone" {
	type primary;
	file "verify-addzone.db.original.signed";
};

zone "verify-axfr" {
	type primary;
	file "verify-axfr.db.signed";
};

zone "verify-csk" {
	type primary;
	file "verify-csk.db.signed";
};

zone "verify-ixfr" {
	type primary;
	file "verify-ixfr.db.signed";
	ixfr-from-differences yes;
	allow-transfer { 10.53.0.3; };
};

zone "verify-reconfig" {
	type primary;
	file "verify-reconfig.db.signed";
};

zone "verify-unsigned" {
	type primary;
	file "verify.db.in";
};

zone "verify-untrusted" {
	type primary;
	file "verify-untrusted.db.signed";
};
