// NS4

options {
	query-source address 10.53.0.4;
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

remote-servers "ns3" {
	10.53.0.3 port @PORT@;
};

zone "retransfer.kasp" {
	type secondary;
	file "retransfer.kasp.db";
	primaries { "ns3"; };
	allow-notify { any; };
	notify no;
};
