{% include "_common/controls.conf.j2" %}

options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { fd92:7065:b8e:ffff::4; };
	recursion yes;
	dnssec-validation no;
	query-source none;
	query-source-v6 fd92:7065:b8e:ffff::4;
};

zone "." {
	type hint;
	file "root.hint";
};
