options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	session-keyfile "session.key";
	listen-on { 10.53.0.1; 127.0.0.1; };
	listen-on-v6 { none; };
	allow-transfer { !10.53.0.1; any; };
	recursion no;
	notify yes;
	dnssec-validation no;
};

include "ddns.key";

{% include "_common/controls.conf.j2" %}

dlz "example one" {
	database "dlopen @TOP_BUILDDIR@/testlib-driver-dlzexternal.@DYLIB@ example.nil";
};

dlz "example two" {
	database "dlopen @TOP_BUILDDIR@/testlib-driver-dlzexternal.@DYLIB@ alternate.nil";
};

dlz "example three" {
	database "dlopen @TOP_BUILDDIR@/testlib-driver-dlzexternal.@DYLIB@ example.org";
};

dlz "example four" {
	// Long zone name to trigger ISC_R_NOSPACE in dns_sdlz_putrr.
        database "dlopen @TOP_BUILDDIR@/testlib-driver-dlzexternal.@DYLIB@ 123456789.123456789.123456789.123456789.123456789.example.foo";
};

dlz "unsearched1" {
	database "dlopen @TOP_BUILDDIR@/testlib-driver-dlzexternal.@DYLIB@ other.nil";
	search no;
};

dlz "unsearched2" {
	database "dlopen @TOP_BUILDDIR@/testlib-driver-dlzexternal.@DYLIB@ zone.nil";
	search no;
};

dlz redzone {
	database "dlopen @TOP_BUILDDIR@/testlib-driver-dlzexternal.@DYLIB@ .";
	search no;
};

zone zone.nil {
	type primary;
	dlz unsearched2;
};

zone "." {
	type redirect;
	dlz redzone;
};

zone "." {
	type primary;
	file "root.db";
};
