// NS1

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	minimal-any no;
	minimal-responses no;
	recursion no;
	notify yes;
	dnssec-validation yes;
	/* test that we can turn off trust-anchor-telemetry */
	trust-anchor-telemetry no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type primary;
	file "root.db.signed";
};

include "trusted.conf";
