{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.7;
	notify-source 10.53.0.7;
	transfer-source 10.53.0.7;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.7; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
	notify yes;
	ixfr-from-differences secondary;
	check-integrity no;
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "primary2" {
	type primary;
	file "primary2.db";
};

zone "secondary" {
	type secondary;
	primaries { 10.53.0.1; };
	file "sec.bk";
};

zone "edns-expire" {
	type secondary;
	primaries { 10.53.0.6; };
	file "edns-expire.bk";
};
