options {
	directory ".";
	query-source address 10.53.0.7;
	notify-source 10.53.0.7;
	transfer-source 10.53.0.7;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.7; };
	listen-on-v6 { fd92:7065:b8e:ffff::7; };
	recursion yes;
	allow-recursion { any; };
	dnssec-validation no;
	deny-answer-aliases {
		"example";
	} except-from {
		"example";
	};
	qname-minimization disabled;  // Regression test for GL #4652
};


key restart16 {
	secret "1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

{% include "_common/controls.conf.j2" %}

view restart16 {
	match-clients { key restart16; none; };
	max-query-restarts 16;

	zone "." {
		type hint;
		file "root.hint";
	};
};

view default {
	zone "." {
		type hint;
		file "root.hint";
	};
};
