// NS2

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-policy "none";
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

/* Inherit dnssec-policy (which is none) */

zone "unsigned.tld" {
	type primary;
	file "unsigned.tld.db";
};

/* Override dnssec-policy */

zone "signed.tld" {
	type primary;
	file "signed.tld.db";
	dnssec-policy "default";
};

/* Primary service for ns3 */

zone "secondary.kasp" {
	type primary;
	file "secondary.kasp.db";
	allow-transfer { 10.53.0.3; };
	notify yes;
};
