{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
	cookie-algorithm siphash24;
	cookie-secret "569d36a6cc27d6bf55502183302ba352";
	cookie-secret "6b300e27a0db46d4b046e4189790fa7d";
	require-server-cookie yes;
};


zone "." {
	type hint;
	file "root.hint";
};
