// NS9 — signed-root upstream for ns8 (advisor cached path) and ns10
// (synth-from-dnssec path)

options {
	port @PORT@;
	listen-on port @PORT@ { 10.53.0.9; };
	pid-file "named.pid";
	dnssec-validation no;
	recursion no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type primary;
	file "root.db.signed";
};

zone "redirect" {
	type primary;
	file "redirect.db";
};
