// NS1

{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	dnssec-validation no;
};


zone "database" {
	type primary;
	database "_builtin empty localhost. @rname | default('hostmaster.isc.org.')@";
};
