options {
	query-source address 10.53.0.6;
	notify-source 10.53.0.6;
	transfer-source 10.53.0.6;
	port @PORT@;
	pid-file "named.pid";
	statistics-file "named.stats";
	session-keyfile "session.key";
	listen-on { 10.53.0.6; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	forward only;
	forwarders { 10.53.0.3; };
	minimal-any no;
	minimal-responses no;
	recursion yes;
	dnssec-validation yes;
	qname-minimization disabled;

	response-policy {
	    zone "policy1" min-update-interval 0;
	    zone "bl.tld2s" policy given;
	} qname-wait-recurse yes
	// add-soa yes # leave add-soa as default for unset test
	nsip-enable yes
	nsdname-enable yes;
};

logging { category rpz { default_debug; }; };

{% include "_common/controls.conf.j2" %}

include "../trusted.conf";

zone "policy1" {
	type secondary;
	primaries { 10.53.0.5; };
	file "empty.db";
	also-notify { 10.53.0.3 port @EXTRAPORT1@; };
	notify-delay 0;
	allow-transfer	{ any; };
};

zone "bl.tld2s." {
    type primary;
    file "bl.tld2s.db";
};
