options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

dnssec-policy "nsec" {
	// no need to change configuration: if no 'nsec3param' is set,
	// NSEC will be used;
};

dnssec-policy "nsec-altalg" {
	keys {
		csk lifetime unlimited algorithm @ALTERNATIVE_ALGORITHM@;
	};
};

dnssec-policy "nsec3" {
	nsec3param;
};

dnssec-policy "optout" {
	nsec3param optout yes;
};

dnssec-policy "nsec3-other" {
	nsec3param iterations 0 optout yes salt-length 8;
};
