options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion yes;
	notify yes;
	serial-query-rate 1; // workaround for KB AA-01213
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

key altkey {
	algorithm @DEFAULT_HMAC@;
	secret "1234abcd8765";
};

view alternate {
	match-clients { key altkey; };

	zone "update.nil" {
		type secondary;
		primaries { 10.53.0.1; };
		file "update.alt.bk";
		allow-transfer { any; };
	};
};

view primary {
	match-clients { any; };

	zone "example.nil" {
		type secondary;
		primaries { 10.53.0.1; };
		file "example.bk";
		allow-transfer { any; };
	};

	zone "update.nil" {
		type secondary;
		primaries { 10.53.0.1; };
		file "update.bk";
		allow-transfer { any; };
	};

	zone "sample" {
		type primary;
		allow-update { any; };
		file "sample.db";
	};
};
