options {
	listen-on port @PORT@ { 10.53.0.2; };
	transfer-source 10.53.0.2;
	pid-file "named.pid";
	recursion no;
};

zone "foo.fr" {
	min-refresh-time 1;
	min-retry-time 1;
	type secondary;
	primaries { 10.53.0.1 port @PORT@; };
};

{% include "_common/controls.conf.j2" %}
