options {
	directory ".";
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	servfail-ttl 0;
	max-recursion-depth 12;
	recursion yes;
	dnssec-validation no;
	min-cache-ttl 60;
	min-ncache-ttl 30;
	max-cache-ttl 120;
	max-ncache-ttl 60;
};

{% include "_common/controls.conf.j2" %}

zone "." { type hint; file "hints.db"; };
