/*
 * DNS64 / RPZ server.
 */

options {
	query-source address 10.53.0.9;
	notify-source 10.53.0.9;
	transfer-source 10.53.0.9;
	port @PORT@;
	pid-file "named.pid";
	statistics-file	"named.stats";
	session-keyfile "session.key";
	listen-on { 10.53.0.9; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	notify yes;
	minimal-any no;
	minimal-responses no;
	recursion yes;
	dnssec-validation no;
	dns64-server "example.localdomain.";
	dns64 64:ff9b::/96 { };
	response-policy {
	    zone "rpz";
	}
	qname-wait-recurse no ;

	notify-delay 0;
};


logging { category rpz { default_debug; }; };
{% include "_common/controls.conf.j2" %}


zone "." { type hint; file "hints"; };

zone "rpz." {
	type primary;
	file "rpz.db";
	notify no;
};
