// NS3

options {
	pid-file "named.pid";
	listen-on port @PORT@ { 10.53.0.3; };
	port @PORT@;
	listen-on-v6 { none; };
	recursion no;
	notify no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "example" {
	type primary;
	masterfile-format raw;
	file "example.db.raw";
};

zone "dynamic" {
	type primary;
	masterfile-format raw;
	file "dynamic.db.raw";
	allow-update { any; };
};
