options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	allow-query { any; };
	recursion no;
	allow-new-zones yes;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}
