options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	session-keyfile "session.key";
	listen-on { 10.53.0.1; 127.0.0.1; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

dyndb sample "@TOP_BUILDDIR@/testlib-driver-sample.@DYLIB@" { ipv4.example.nil. in-addr.arpa. };
dyndb sample2 "@TOP_BUILDDIR@/testlib-driver-sample.@DYLIB@" { ipv6.example.nil. 8.b.d.0.1.0.0.2.ip6.arpa. };
