options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
	notify no;
};

{% include "_common/controls.conf.j2" %}

view "in" {
	allow-transfer { any; };

	zone "example." {
		type primary;
		file "example-in.db";
	};

	zone "broken1." {
		type primary;
		file "broken1.db";
	};

	zone "broken2." {
		type primary;
		file "broken2.db";
	};

	zone "broken3." {
		type primary;
		file "broken3.db";
	};

	zone "broken4." {
		type primary;
		file "broken4.db";
	};

	zone "broken5." {
		type primary;
		file "broken5.db";
	};
};
