options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	dnssec-validation no;
	send-report-channel "example.rad";
};

{% include "_common/controls.conf.j2" %}

view main in {
	allow-transfer { any; };

	zone example.net {
		type primary;
		file "example.net.db";
	};

	zone example.com {
		type primary;
		file "example.com.db";
		send-report-channel "rad.example.net";
	};

	zone rad {
		type primary;
		file "rad.db";
	};

	zone example.rad {
		type primary;
		file "example.rad.db";
		log-report-channel yes;
	};
};
