options {
	port @PORT@;
	pid-file "named.pid";
	session-keyfile "session.key";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
};

key rndc_key {
	secret "1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

key secondkey {
	secret "abcd1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

/* deliberately not _common/controls.conf.j2: multi-key control channel is the test subject */
controls {
	inet 10.53.0.2 port @CONTROLPORT@ allow { any; } keys { rndc_key; secondkey; };
};


zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "nil" {
	type primary;
	update-policy local;
	file "nil.db";
	ixfr-from-differences yes;
};

zone "other" {
	type primary;
	update-policy local;
	file "other.db";
};

zone "static" {
	type primary;
	file "static.db";
};

zone "incl" {
	type primary;
	file "incl.db";
};
