options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	dnssec-validation no;
};

key secondkey {
	secret "abcd1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

{% include "_common/controls.conf.j2" %}

view all {
	match-clients { any; };

	recursion no;

	zone "." {
		type hint;
		file "../../_common/root.hint";
	};
};

view none {
	match-clients { none; };
};
