{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	dnssec-validation no;
	recursion no;
	notify no;
	tcp-initial-timeout 20;
	tcp-idle-timeout 50;
	tcp-keepalive-timeout 70;
	max-transfer-time-out 5; /* minutes */
	max-transfer-idle-out 1; /* minutes */
};

zone "." {
	type primary;
	file "root.db";
};

zone "example." {
	type primary;
	file "example.db";
	check-integrity no;
};
