options {
	query-source address 10.53.0.8;
	notify-source 10.53.0.8;
	transfer-source 10.53.0.8;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.8; };
	listen-on-v6 { none; };
	forwarders { 10.53.0.2; };	// returns referrals
	forward first;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "root.db";
};

zone "sub.local.tld" {
	type primary;
	file "sub.local.tld.db";
};
