options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	pid-file "named.pid";
	session-keyfile "session.key";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	minimal-responses no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "local.nil" {
	type primary;
	file "local.db";
	update-policy local;
};
