options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion yes;
	dnssec-validation no;
	notify yes;
};

{% include "_common/controls.conf.j2" %}

key tsigzone. {
	algorithm @DEFAULT_HMAC@;
	secret "1234abcd8765";
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "example" {
	type secondary;
	primaries { 10.53.0.2; };
	file "example.bk";
};

zone "primary" {
	type secondary;
	transfer-source 10.53.0.3;
	primaries { 10.53.0.6; };
	file "primary.bk";
};

server 10.53.0.2 {
	keys { tsigzone.; };
};

zone "tsigzone" {
	type secondary;
	primaries { 10.53.0.2; };
	file "tsigzone.bk";
	allow-transfer { key tsigzone.; };
};

zone "mapped" {
	type secondary;
	primaries { 10.53.0.2; };
	masterfile-format raw;
	file "mapped.bk";
};

zone "xfer-stats" {
	type secondary;
	primaries { 10.53.0.1; };
	file "xfer-stats.bk";
};
