{% set wrongoption = wrongoption | default(False) %}

options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
};

{% include "_common/controls.conf.j2" %}

view "default" {
	zone "example." {
		type primary;
		file "example.db";
	};
	/*
	 * failure to reconfig must occurs during configuration of the views
	 */
	{% if wrongoption %}
		forwarders port 9999999 { 127.0.0.1; };
	{% endif %}
};
