// validating resolver

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation yes;
	synth-from-dnssec yes;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "f004.test" {
	type static-stub;
	server-addresses { 10.53.0.1; };
};

zone "f007.test" {
	type static-stub;
	server-addresses { 10.53.0.3; };
};

zone "f023.test" {
	type static-stub;
	server-addresses { 10.53.0.1; };
};

zone "evil.f023.test" {
	type static-stub;
	server-addresses { 10.53.0.1; };
};

trust-anchors {
	f004.test. static-key 257 3 13 "@DNSKEY@";
	f007.test. static-key 257 3 13 "@DNSKEY@";
	f023.test. static-key 257 3 13 "@DNSKEY@";
};
