{% include "_common/controls.conf.j2" %}

tls local {
	key-file "../self-signed-key.pem";
	cert-file "../self-signed-cert.pem";
};

options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	tls-port @TLSPORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on tls local { 10.53.0.5; };             // DoT
	listen-on-v6 { none; };
	recursion no;
	notify no;
	ixfr-from-differences yes;
	check-integrity no;
	dnssec-validation no;
};


zone "." {
	type hint;
	file "../../_common/root.hint";
};

tls tls-v1.2 {
	protocols { TLSv1.2; };
	prefer-server-ciphers no;
};

zone "example" {
	type secondary;
	primaries { 10.53.0.1 tls tls-v1.2; };
	file "example.db";
	allow-transfer { any; };
};

zone "example-aes-128" {
	type secondary;
	primaries port @EXTRAPORT1@ { 10.53.0.1 tls tls-v1.2; };
	file "example-aes-128.db";
	allow-transfer { any; };
};

zone "example-aes-256" {
	type secondary;
	primaries port @EXTRAPORT2@ { 10.53.0.1 tls tls-v1.2; };
	file "example-aes-256.db";
	allow-transfer { any; };
};

zone "example-chacha-20" {
	type secondary;
	primaries port @EXTRAPORT3@ { 10.53.0.1 tls tls-v1.2; };
	file "example-chacha-20.db";
	allow-transfer { any; };
};
