options {
	directory ".";
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	servfail-ttl 0;
	qname-minimization disabled;
	max-recursion-depth 100;
	max-recursion-queries 50;
	recursion yes;
	dnssec-validation no;
};


{% include "_common/controls.conf.j2" %}

zone "." { type hint; file "hints.db"; };
