// NS2

acl rfc1918 { 10/8; 192.168/16; 172.16/12; };

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion yes;
	notify yes;
	dnssec-validation no;

};


{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "." {
	type redirect;
	file "redirect.db";
	allow-query { !10.53.0.4; any; };
};

zone "example.nil" {
	type primary;
	file "example.db";
};
