// NS9

options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { none; };
	listen-on-v6 { fd92:7065:b8e:ffff::9; };
	recursion yes;
	recursive-clients 0; // regression test for [GL #4987]
	dnssec-validation no;
	dual-stack-servers { fd92:7065:b8e:ffff::7; };
	qname-minimization off;
	minimal-any no;
};


key rndc_key {
	secret "1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

/* deliberately not _common/controls.conf.j2: IPv6 control channel; the shared template is IPv4-only */
controls {
	inet fd92:7065:b8e:ffff::9 port @CONTROLPORT@ allow { any; } keys { rndc_key; };
};

zone "." {
	type hint;
	file "root.hint";
};
