// NS5

options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation yes;
	querylog yes;
	prefetch 4 10;
	responselog yes;
	request-nsid yes;
	request-zoneversion yes;
	minimal-any no;

	// "checking that removal of a delegation is honoured"
	// test has a delegation with a TTL of 5 seconds, so let's
	// min a min below it.
	min-delegation-ttl 3;
};

// Don't break tests which depend on ans10 by requesting
// zoneversion or nsid
server 10.53.0.10 {
	request-nsid no;
	request-zoneversion no;
};

include "trusted.conf";

server 10.53.0.7 {
	edns-version 0;
};

zone "." {
	type hint;
	file "root.hint";
};

zone "moves" {
	type primary;
	file "moves.db";
};

zone "child.server" {
	type primary;
	file "child.server.db";
};

include "trusted.conf";

{% include "_common/controls.conf.j2" %}
