options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
	notify no;
	minimal-responses no;
	version none;  // make statistics independent of the version number
	sig-signing-signatures 100;
};

statistics-channels { inet 10.53.0.2 port @EXTRAPORT1@ allow { localhost; }; };

{% include "_common/controls.conf.j2" %}

dnssec-policy "dnssec" {
	keys {
		ksk lifetime unlimited algorithm @DEFAULT_ALGORITHM@;
		zsk lifetime unlimited algorithm @DEFAULT_ALGORITHM@;
	};
};

dnssec-policy "manykeys" {
	inline-signing no;
	keys {
		ksk lifetime unlimited algorithm 8;
		zsk lifetime unlimited algorithm 8;
	};
};

zone "example" {
	type primary;
	file "example.db";
	allow-transfer { any; };
};

zone "dnssec" {
	type primary;
	file "dnssec.db.signed";
	allow-update { any; };
	zone-statistics full;
	dnssec-policy "dnssec";
};

zone "manykeys" {
	type primary;
	file "manykeys.db.signed";
	allow-update { any; };
	zone-statistics full;
	dnssec-policy "manykeys";
};
