{% set zone_names = zone_names | default([]) %}

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
	notify no;
};

{% include "_common/controls.conf.j2" %}

{% for name in zone_names %}
zone "@name@" {
    type primary;
    file "@name@.db";
    dnssec-policy default;
    inline-signing yes;
};
{% endfor %}
