tls tls-forward-secrecy {
    protocols { TLSv1.2; };
    ciphers "HIGH:!kRSA:!aNULL:!eNULL:!RC4:!3DES:!MD5:!EXP:!PSK:!SRP:!DSS:!SHA1:!SHA256:!SHA384";
    prefer-server-ciphers yes;
    key-file "../CA/certs/srv02.crt01.example.nil.key";
    cert-file "../CA/certs/srv02.crt01.example.nil.pem";
    dhparam-file "../dhparam3072.pem";
};

tls tls-forward-secrecy-mutual-tls {
    protocols { TLSv1.2; };
    ciphers "HIGH:!kRSA:!aNULL:!eNULL:!RC4:!3DES:!MD5:!EXP:!PSK:!SRP:!DSS:!SHA1:!SHA256:!SHA384";
    prefer-server-ciphers yes;
    key-file "../CA/certs/srv02.crt01.example.nil.key";
    cert-file "../CA/certs/srv02.crt01.example.nil.pem";
    dhparam-file "../dhparam3072.pem";
    ca-file "../CA/CA.pem";
};

tls tls-expired {
    protocols { TLSv1.2; };
    ciphers "HIGH:!kRSA:!aNULL:!eNULL:!RC4:!3DES:!MD5:!EXP:!PSK:!SRP:!DSS:!SHA1:!SHA256:!SHA384";
    prefer-server-ciphers yes;
    key-file "../CA/certs/srv02.crt02-expired.example.nil.key";
    cert-file "../CA/certs/srv02.crt02-expired.example.nil.pem";
    dhparam-file "../dhparam3072.pem";
};
