options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.7; };
	listen-on-v6 { none; };
	dnssec-validation no;
};

key int {
	algorithm @DEFAULT_HMAC@;
	secret "FrSt77yPTFx6hTs4i2tKLB9LmE0=";
};

key ext {
	algorithm @DEFAULT_HMAC@;
	secret "FrSt77yPTFx6hTs4i2tKLB9LmE0=";
};

{% include "_common/controls.conf.j2" %}

view internal {
	match-clients { key "int"; };

	zone "test" {
		type primary;
		update-policy { grant int zonesub any; };
		file "test.db";
		ixfr-from-differences yes;
	};
};

view external {
	match-clients { key "ext"; };

	zone "test" {
		in-view internal;
	};
};
