options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { fd92:7065:b8e:ffff::2; };
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "ipv4-only-servers" {
	type secondary;
	primaries { 10.53.0.3; };
};

zone "ipv6-only-servers" {
	type secondary;
	primaries { fd92:7065:b8e:ffff::2; };
};
