{% set broken_ns = broken_ns | default(False) %}

$TTL 300
. 			IN SOA	gson.nominum.com. a.root.servers.nil. (
				2000042100   	; serial
				600         	; refresh
				600         	; retry
				1200    	; expire
				600       	; minimum
				)
.			NS	a.root-servers.nil.
a.root-servers.nil.	A	10.53.0.1

test.			NS	ns.test.

{% if broken_ns %}
ns.test.		A	10.53.0.5
{% else %}
ns.test.		A	10.53.0.2
{% endif %}
