$TTL 300
$ORIGIN secondary.

secondary.		IN	SOA  mname1. . (
			1       ; serial
			20      ; refresh (20 seconds)
			20      ; retry (20 seconds)
			1814400 ; expire (3 weeks)
			3600    ; minimum (1 hour)
			)

			NS	ns2
ns2			A	10.53.0.2

scanner			A	10.53.0.2
bad-scanner		A	6.6.6.6

model2			NS	ns3
			NS	ns4

ns3.model2		A	10.53.0.3
ns4.model2		A	10.53.0.4

model2._dsync		DSYNC	CDS NOTIFY @PORT@ scanner
model2._dsync		DSYNC	CSYNC NOTIFY @CONTROLPORT@ bad-scanner ; ignored, unknown rrtype
model2._dsync		DSYNC	DSYNC NOTIFY @CONTROLPORT@ bad-scanner ; ignored, bad rrtype
