{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	recursion yes;
	notify no;
	hostname "this.is.a.test.of.hostname";
	server-id "this.is.a.test.of.server-id";
	version "this is a test of version";
	dnssec-validation no;
};
