{% set ns4_as_secondary_for_nil = ns4_as_secondary_for_nil | default(False) %}
options {
	query-source address 10.53.0.4;
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
	notify yes;
	blackhole { none; };
};

key unused_key. {
	secret "1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

key tsig_key. {
	secret "LSAnCU+Z";
	algorithm @DEFAULT_HMAC@;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type primary;
	file "root.db";
};

{% if ns4_as_secondary_for_nil %}
zone "nil" {
    type secondary;
    file "nil.db";
    primaries { 10.53.0.5 key tsig_key; };
};
{% endif %}
