options {
	query-source address 10.53.0.4;
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
	notify no;
	minimal-responses no;
	version none;  // make statistics independent of the version number
};

statistics-channels { inet 10.53.0.4 port @EXTRAPORT1@ allow { localhost; }; };

{% include "_common/controls.conf.j2" %}

zone "example2" {
	type primary;
	file "example2.db";
	allow-transfer { any; };
};
