options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion yes;
	dnssec-validation no;
	notify yes;
};

{% include "_common/controls.conf.j2" %}

zone "example" {
	type secondary;
	file "example.bk";
	primaries { 10.53.0.1; };
};

zone "example2" {
	type secondary;
	file "example2.bk";
	primaries { 10.53.0.1; };
};

zone "example3" {
	type secondary;
	file "example3.bk";
	primaries { 10.53.0.1; };
};
