options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	max-cache-size 8000;
};

statistics-channels {
	inet 10.53.0.1 port @EXTRAPORT1@;
};

{% include "_common/controls.conf.j2" %}
