// NS2

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	notify yes;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

template primary {
	type primary;
	file "$name.db";
};

zone "example" {
	template primary;
};

zone "missing" {
	template primary;
};

zone "different" {
	template primary;
	initial-file "example.db";
	file "alternate.db";
};

zone "initial" {
	type primary;
	file "copied.db";
	initial-file "example.db";
};

zone "present" {
	type primary;
	file "present.db";
	initial-file "example.db";
};

template secondary {
	type secondary;
	primaries { 10.53.0.1; };
	file "$name.db";
};

zone "extra1" {
	template secondary;
};

zone "extra2" {
	template secondary;
};
