options {
	query-source address 10.53.0.3;
	query-source-v6 address fd92:7065:b8e:ffff::3;
	notify-source 10.53.0.3;
	notify-source-v6 fd92:7065:b8e:ffff::3;
	transfer-source 10.53.0.3;
	transfer-source-v6 fd92:7065:b8e:ffff::3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { fd92:7065:b8e:ffff::3; };
	forwarders { 10.53.0.6; };
	dnssec-validation yes;
};

include "trusted.conf";

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "root2.db";
};
