// NS5

options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	recursion yes;
	qname-minimization disabled;
	querylog yes;
	resolver-query-timeout 30000; # 30 seconds
	dnssec-validation no;
	disable-empty-zone 10.in-addr.arpa;
	fetches-per-zone 40;

	// Some domains in this tests have TTL of 2, so let's pick
	// a value below it.
	min-delegation-ttl 1;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "in-addr.arpa" {
	type primary;
	file "in-addr.arpa.db";
};
