options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { fd92:7065:b8e:ffff::5; };
	recursion yes;
	dnssec-validation no;
	notify yes;
	dns64 64:ff9b::/96 {
		clients { any; };
		exclude { ::1/128; };
		mapped { any; };
	};
	minimal-responses no;
};

plugin query "@TOP_BUILDDIR@/filter-aaaa.@DYLIB@" {
	filter-aaaa-on-v4 break-dnssec;
	filter-aaaa { any; };
};

{% include "_common/controls.conf.j2" %}

zone "." { type hint; file "hints"; };

include "trusted.conf";
