options {
	query-source address 10.53.0.10;
	notify-source 10.53.0.10;
	transfer-source 10.53.0.10;
	port @PORT@;
	pid-file "named.pid";
	session-keyfile "session.key";
	listen-on { 10.53.0.10; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	notify no;
	minimal-any no;
	minimal-responses no;
	recursion yes;
	dnssec-validation yes;
};
{% include "_common/controls.conf.j2" %}

include "../trusted.conf";
zone "." { type hint; file "hints"; };

# grafted on zones using stub and static-stub
zone "stub-nomatch."		    {type primary; file "stub.db"; };
zone "static-stub-nomatch."	    {type primary; file "stub.db"; };
