options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	recursion no;
	dnssec-validation no;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	notify no;
	minimal-responses yes;
};

{% include "_common/controls.conf.j2" %}

zone "." {
        type primary;
        file "root.db";
};

zone "rt.example" {
	type primary;
	file "rt.db";
};

zone "naptr.example" {
	type primary;
	file "naptr.db";
};

zone "rt2.example" {
	type primary;
	file "rt2.db";
};

zone "naptr2.example" {
	type primary;
	file "naptr2.db";
};

zone "nid.example" {
	type primary;
	file "nid.db";
};
