options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	recursion no;
	allow-recursion { none; };
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

view internal {
        match-destinations { 10.53.0.3; };
        zone "." {
                type hint;
                file "../../_common/root.hint";
        };
        recursion yes;
	allow-query{ any; };
};
