// NS5

options {
	port @PORT@;
	listen-on port @PORT@ { 10.53.0.5; };
	pid-file "named.pid";
	nxdomain-redirect signed;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
        type primary;
        file "root.db.signed";
};

// An unsigned zone that ns6 has a delegation for.
zone "unsigned." {
        type primary;
        file "unsigned.db";
};
