dnssec-policy "rsasha1" {
	dnskey-ttl 1234;

	keys {
		ksk key-directory lifetime P10Y algorithm 5;
		zsk key-directory lifetime P5Y  algorithm 5;
		zsk key-directory lifetime P1Y  algorithm 5 2000;
	};
};

dnssec-policy "rsasha1-nsec3" {
	dnskey-ttl 1234;

	keys {
		ksk key-directory lifetime P10Y algorithm 7;
		zsk key-directory lifetime P5Y  algorithm 7;
		zsk key-directory lifetime P1Y  algorithm 7 2000;
	};
};
