{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.6;
	notify-source 10.53.0.6;
	transfer-source 10.53.0.6;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.6; };
	listen-on-v6 { none; };
	dnssec-validation no;
	recursion no;
};

zone "stale" IN {
	type primary;
	notify no;
	file "stale.db";
};

zone "serve.stale" IN {
	type primary;
	notify no;
	file "serve.stale.db";
};
