options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
	notify yes;
	ixfr-from-differences yes;
	check-integrity no;
};

{% include "_common/controls.conf.j2" %}

key tsigzone. {
	algorithm @DEFAULT_HMAC@;
	secret "1234abcd8765";
};

acl tzkey {
	key tsigzone.;
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "example" {
	type primary;
	file "example.db";
};

zone "tsigzone" {
	type primary;
	file "tsigzone.db";
	allow-transfer { tzkey; };
};

remote-servers "ns1" port @PORT@ source 10.53.0.2 {
	10.53.0.1;
};

zone "secondary" {
	type secondary;
	file "sec.db";
	primaries { ns1; };
	masterfile-format text;
};

zone "mapped" {
	type secondary;
	file "mapped.db";
	masterfile-format text;
	primaries { 10.53.0.100; };
};

zone "dot-fallback" {
	type secondary;
	file "dot-fallback.db";
	primaries {
		10.53.0.1 tls ephemeral;
		10.53.0.1;
	};
};
