// NS2

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { fd92:7065:b8e:ffff::2; };
	recursion no;
	dnssec-validation no;
	geoip-directory "../data";
};

{% include "_common/controls.conf.j2" %}

view one {
	match-clients { geoip region CA; };
	zone "example" {
		type primary;
		file "example1.db";
	};
};

view three {
	match-clients { geoip region OK; };
	zone "example" {
		type primary;
		file "example3.db";
	};
};

view four {
	match-clients { geoip region VA; };
	zone "example" {
		type primary;
		file "example4.db";
	};
};

view five {
	match-clients { geoip region GA; };
	zone "example" {
		type primary;
		file "example5.db";
	};
};

view six {
	match-clients { geoip region CO; };
	zone "example" {
		type primary;
		file "example6.db";
	};
};

view seven {
	match-clients { geoip region AK; };
	zone "example" {
		type primary;
		file "example7.db";
	};
};

view none {
	match-clients { any; };
	zone "example" {
		type primary;
		file "example.db.in";
	};
};
