{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.8;
	notify-source 10.53.0.8;
	transfer-source 10.53.0.8;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.8; };
	listen-on-v6 { none; };
	dnssec-validation no;
	rate-limit {};
	require-server-cookie yes;
};


server 10.53.0.7 { require-cookie yes; };

zone "example" {
	type primary;
	file "example.db";
};
