// NS2

acl rfc1918 { 10/8; 192.168/16; 172.16/12; };

options {
	query-source address 10.53.0.2;	/* note this is not 10.53.0.4 */
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	recursion yes;
	notify yes;
	dnssec-validation no;
	nxdomain-redirect "redirect";
};


{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "root.hint";
};

zone "example" {
	type primary;
	file "example.db";
};
