{% set filtertype = filtertype | default("aaaa") %}
{% set family = family | default("v4") %}

options {
	query-source address 10.53.0.3;
	query-source-v6 address fd92:7065:b8e:ffff::3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { fd92:7065:b8e:ffff::3; };
	recursion yes;
	dnssec-validation yes;
	notify yes;
	minimal-responses no;
};

{% if family == "v6" %}
        plugin query "@TOP_BUILDDIR@/filter-@filtertype@.@DYLIB@" {
                filter-@filtertype@-on-v6 break-dnssec;
                filter-@filtertype@ { fd92:7065:b8e:ffff::3; };
        };
{% else %}
        plugin query "@TOP_BUILDDIR@/filter-@filtertype@.@DYLIB@" {
                filter-@filtertype@-on-v4 break-dnssec;
                filter-@filtertype@ { 10.53.0.3; };
        };
{% endif %}

{% include "_common/controls.conf.j2" %}

zone "." { type hint; file "hints"; };

include "trusted.conf";
