options {
	query-source address 10.53.0.4;
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	notify yes;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

view "primary" {
	ixfr-from-differences yes;
	request-ixfr yes;
	zone "test" IN {
		type secondary;
		file "mytest.db";
		primaries { 10.53.0.3; };
		max-ixfr-ratio unlimited;
	};
	zone "sub.test" IN {
		type secondary;
		file "subtest.db";
		request-ixfr no;
		primaries { 10.53.0.3; };
	};
};
