zone 192.in-addr.arpa {
	type primary;
	file "file";
	plugin query "../../../plugins/.libs/synthrecordwrong.so" {
		prefix "dynamic-";
		allow-synth {
			192.168.1.0/24;
			192.180.0.0/18;
		};
		origin example.;
		ttl 60;
	};
};
