options {
	query-source address 10.53.0.1;
	query-source-v6 address fd92:7065:b8e:ffff::1;
	notify-source 10.53.0.1;
	notify-source-v6 fd92:7065:b8e:ffff::1;
	transfer-source 10.53.0.1;
	transfer-source-v6 fd92:7065:b8e:ffff::1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { fd92:7065:b8e:ffff::1; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type primary;
	file "root.db.signed";
};

zone "example1." {
	type primary;
	file "example.db";
};

zone "example2." {
	type primary;
	file "example.db";
};

zone "example3." {
	type primary;
	file "example.db";
};

zone "example4." {
	type primary;
	file "example.db";
};

zone "example5." {
	type primary;
	file "example.db";
};

zone "sld.tld" {
	type primary;
	file "sld.tld.db";
};

/* A forward zone without forwarders. */
zone "example6" {
	type forward;
};

zone "example8." {
	type primary;
	file "example.db";
};

zone "example9." {
	type primary;
	file "example.db";
};

zone "diditwork.net" {
	type primary;
	file "diditwork.net.db";
};

zone "spoofed.net" {
	type primary;
	file "spoofed.net.db";
};

zone "sub.local.net" {
	type primary;
	file "sub.local.net.db";
};

zone "net.example.lll" {
	type master;
	file "net.example.lll";
};
