// NS1

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	dnssec-validation yes;
};

{% include "_common/controls.conf.j2" %}

{% if FEATURE_LIBXML2 == "1" or FEATURE_JSON_C == "1" %}
statistics-channels {
	inet 10.53.0.1 port @EXTRAPORT1@ allow { any; };
};
{% endif %}

zone "." {
	type primary;
	file "root.db.signed";
};

zone "example" {
	type primary;
	file "example.db.signed";
};

zone "insecure.example" {
	type primary;
	file "insecure.example.db.signed";
};

zone "dnamed" {
	type primary;
	file "dnamed.db.signed";
};

zone "minimal" {
	type primary;
	file "minimal.db.signed";
};

zone "no-apex-covering" {
	type primary;
	file "no-apex-covering.db.signed";
};

zone "soa-without-dnskey" {
	type primary;
	file "soa-without-dnskey.db.signed";
};

include "trusted.conf";
