{% set use_ssu = use_ssu | default(False) %}

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "example" {
	type primary;
	file "example.db";
{% if use_ssu %}
	update-policy { grant * self * A; };
{% else %}
	allow-update { any; };
{% endif %}
};
