options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	directory ".";
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	recursion yes;
	notify yes;
	dnstap-identity "ns5";
	dnstap-version "xxx";
	dnstap-output file "dnstap.out";
	dnstap { all; };
	send-cookie no;
	require-server-cookie no;
	dnssec-validation no;
	qname-minimization disabled;
	forwarders { 10.53.0.3; };
	forward only;
};

{% include "_common/controls.conf.j2" %}
