options {
	query-source address 10.53.0.9;
	notify-source 10.53.0.9;
	transfer-source 10.53.0.9;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.9; };
	listen-on-v6 { none; };
	dnssec-validation no;
	edns-udp-size 1232;
};

{% include "_common/controls.conf.j2" %}

server 10.53.0.10 {
	edns no;
};

server 10.53.0.11 {
	edns no;
};

zone "." {
	type hint;
	file "root.db";
};

zone "attacksecuredomain.net." {
	type forward;
	forward only;
	forwarders { 10.53.0.10; };
};

zone "attacksecuredomain.net2." {
	type forward;
	forward only;
	forwarders { 10.53.0.10; };
};

zone "attacksecuredomain.net3." {
	type forward;
	forward only;
	forwarders { 10.53.0.11; };
};

zone "local.net." {
	type primary;
	file "local.net.db";
	forwarders {};
};
