dnssec-policy "unlimited-lifetime" {
	keys {
		csk lifetime unlimited algorithm @DEFAULT_ALGORITHM@;
	};
};
dnssec-policy "short-lifetime" {
	keys {
		csk lifetime P6M algorithm @DEFAULT_ALGORITHM@;
	};
};

dnssec-policy "long-lifetime" {
	keys {
		csk lifetime P1Y algorithm @DEFAULT_ALGORITHM@;
	};
};
