{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.8;
	notify-source 10.53.0.8;
	transfer-source 10.53.0.8;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.8; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
	notify no;
	transfer-message-size 1024;
};

key key1. {
	algorithm @DEFAULT_HMAC@;
	secret "1234abcd8765";
};

acl tzkey {
	key key1.;
};

zone "example." {
	type primary;
	file "example.db";
	allow-transfer { tzkey; };
};
