// NS4

options {
	pid-file "named.pid";
	listen-on port @PORT@ { 10.53.0.4; };
	port @PORT@;
	listen-on-v6 { none; };
	recursion no;
	notify no;
	session-keyfile "session.key";
	servfail-ttl 0;
	dnssec-validation no;

	/* Ridicously low on purpose */
	max-records-per-type 1;
	max-types-per-name 9;
};

{% include "_common/controls.conf.j2" %}

/*
 * The template zone is fine, but when adding the DNSSEC records to the apex,
 * the max-types-per-name will be exceeded, meaning signing should fail.
 */
zone "kasp-max-types-per-name" {
	type primary;
	file "kasp-max-types-per-name.db.raw";
	masterfile-format raw;
	dnssec-policy "default";
	inline-signing no;
	allow-update { any; };
	allow-transfer { any; };
};
