view someview {
	zone "foo.bar" {
		plugin query "@TOP_BUILDDIR@/testlib-driver-syncplugin" {
			rcode servfail;
		};
		type primary;
		file "foo.bar.db";
	};
	plugin query "@TOP_BUILDDIR@/testlib-driver-syncplugin.@DYLIB@" {
		rcode servfail;
	};
};
