options {
	notify-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	notify explicit;

	/*
	 * We don't want the server to sign the zone here (which is the
	 * default), as we want to keep the zone partially signed. All which
	 * matters is that if +dnssec is set (EDNS flag DO) then RRSIG
	 * associated to the QTYPE, if present, is provided in the answer.
	 */
	dnssec-policy none;
};

{% include "_common/controls.conf.j2" %}

zone "test" {
	file "test.db";
	type primary;
};
