options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
};

key rndc_key {
	secret "1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

/* deliberately not _common/controls.conf.j2: read-only control channel is the test subject */
controls {
	inet 10.53.0.5 port @CONTROLPORT@ allow { any; } keys { rndc_key; } read-only yes;
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};
