// NS5

options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion yes;
	dnssec-validation yes;
	send-cookie no;
};

{% include "_common/controls.conf.j2" %}

trust-anchors {
    evil.test. static-key 257 3 13 "yh1W7zgrqOsAZdKAh597SI7F2ye4ReiLmBNsDg+TDLJQ+3C2fXfrsQyY MvA+hmzTQdKX24zlVlD3YAVA6+VmrQ==";
};

zone "evil.test" {
    type forward;
    forward only;
    forwarders { 10.53.0.7 port @PORT@; };
};
