options {
	listen-on port @PORT@ { 10.53.0.1; };
	transfer-source 10.53.0.1;
	pid-file "named.pid";
	recursion no;
	also-notify { 10.53.0.2 port @PORT@; };
	notify-source 10.53.0.1;
};

zone "foo.fr" {
	type primary;
	allow-transfer{ 10.53.0.2; };
	file "foo.fr.db";
};

{% include "_common/controls.conf.j2" %}
