options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

key "ddns-key" {
	algorithm @DEFAULT_HMAC@;
	secret "c2VjcmV0";
};

zone "example" {
	type primary;
	file "example.db";
	update-policy {
		grant ddns-key subdomain example. CNAME A TXT(3);
	};
};

zone "maxrrperset" {
	type primary;
	file "maxrrperset.db";
	allow-update { any; };
	max-records-per-type 10;
};
