options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
	notify yes;
	check-integrity no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type primary;
	file "root.db";
};

zone "ignore.example" {
	type primary;
	file "ignore.example.db";
	check-names ignore;
};

zone "warn.example" {
	type primary;
	file "warn.example.db";
	check-names warn;
};

zone "fail.example" {
	type primary;
	file "fail.example.db";
	check-names fail;
};

zone "ignore.update" {
	type primary;
	file "ignore.update.db";
	allow-update { any; };
	check-names ignore;
};

zone "warn.update" {
	type primary;
	file "warn.update.db";
	allow-update { any; };
	check-names warn;
};

zone "fail.update" {
	type primary;
	file "fail.update.db";
	allow-update { any; };
	check-names fail;
};
