key-store "ksk" {
	directory "ksk";
};

key-store "zsk" {
	directory "zsk";
};

dnssec-policy "keystores-kz" {
	keys {
		ksk key-store "ksk" lifetime unlimited algorithm ECDSAP256SHA256;
		zsk key-store "zsk" lifetime unlimited algorithm ECDSAP256SHA256;
	};
};

zone "bad-keystores.kz.example" {
	type primary;
	file "bad-keystores.kz.example.db";
	dnssec-policy "keystores-kz";
};
