{% if trust_anchors is defined %}
include "trusted.conf";
{% set dnssec_validation = "yes" %}
{% else %}
{% set dnssec_validation = "auto" %}
{% endif %}


options {
        query-source address 10.53.0.3;
        notify-source 10.53.0.3;
        transfer-source 10.53.0.3;
        port @PORT@;
        pid-file "named.pid";
        listen-on { 10.53.0.3; };
        listen-on-v6 { none; };
        allow-transfer { any; };
        recursion yes;
        dnssec-validation @dnssec_validation@;
        notify-cfg CDS { notify yes; };
};

{% include "_common/controls.conf.j2" %}

zone "." {
        type hint;
        file "../../_common/root.hint";
};
