options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
	deny-answer-addresses { 192.0.2.0/24; 2001:db8:beef::/48; }
		 except-from { "example.org"; };
	deny-answer-aliases { "example.org"; }
		except-from { "goodcname.example.net";
			      "gooddname.example.net"; };
	allow-query {!10.53.0.8; any; };
	max-zone-ttl unlimited;
	resolver-query-timeout 5000; # 5 seconds
	attach-cache "globalcache";
	max-recursion-queries 100;
	request-zoneversion yes;
	minimal-any no;
};


server 10.53.0.3 {
	tcp-only yes;
};

server 10.42.23.3/32 {
     notify-source 10.42.22.1;
     query-source address 10.42.22.1;
     transfer-source 10.42.22.1;
};

server fd92:7065:b8e:ffff::1000 {
     notify-source-v6 fd92:7065:b8e:ffff::1001;
     query-source-v6 address fd92:7065:b8e:ffff::1001;
     transfer-source-v6 fd92:7065:b8e:ffff::1001;
};

view "default" {
	zone "." {
		type hint;
		file "root.hint";
	};
};

view "alternative" {
	zone "." {
		type hint;
		file "root.hint";
	};
};

{% include "_common/controls.conf.j2" %}
