// NS2

acl blocking {
	geoip db country country AU;
};

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { fd92:7065:b8e:ffff::2; };
	recursion no;
	dnssec-validation no;
	geoip-directory "../data";
	blackhole { blocking; };
};

{% include "_common/controls.conf.j2" %}
