options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	recursion no;
	dnssec-validation yes;
	notify yes;
	minimal-responses no;
};

trust-anchors { };

{% include "_common/controls.conf.j2" %}

plugin query "@TOP_BUILDDIR@/testlib-driver-syncplugin" {
	rcode noerror;
	source view;
};

zone "example.com" {
	type primary;
	file "example.db";
};

zone "example2.com" {
	type primary;
	file "example.db";
	plugin query "@TOP_BUILDDIR@/testlib-driver-syncplugin" {
		rcode servfail;
		source zone;
	};
};

zone "example3.com" {
	type primary;
	file "example.db";
	plugin query "@TOP_BUILDDIR@/testlib-driver-syncplugin" {
		rcode notimp;
		source zone;
	};
};

template exampletmpl {
	type primary;
	file "$name.db";
	plugin query "@TOP_BUILDDIR@/testlib-driver-syncplugin" {
		rcode notauth;
		firstlbl "skipfoo";
		source zone;
	};
};

zone "example4.com" {
	template exampletmpl;
	plugin query "@TOP_BUILDDIR@/testlib-driver-syncplugin" {
		rcode notzone;
		source zone;
	};
};
