key foo {
	secret "aaaaaaaaaaaa";
	algorithm @DEFAULT_HMAC@;
};

server 10.53.0.10 {
	keys foo;
};

{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
	deny-answer-addresses { 192.0.2.0/24; 2001:db8:beef::/48; }
		 except-from { "example.org"; };
	deny-answer-aliases { "example.org"; }
		except-from { "goodcname.example.net";
			      "gooddname.example.net"; };
	allow-query {!10.53.0.8; any; };
	send-cookie yes;
	nocookie-udp-size 512;
};


zone "." {
	type hint;
	file "root.hint";
};

zone "example" {
	type primary;
	file "example.db";
};
