zone "rsasha1.kasp" {
	type primary;
	file "rsasha1.kasp.db";
	dnssec-policy "rsasha1";
};

zone "rsasha1-nsec3.kasp" {
	type primary;
	file "rsasha1-nsec3.kasp.db";
	dnssec-policy "rsasha1-nsec3";
};
