logging {
	channel basic {
		file "named.run";
		severity debug 999;
		print-time yes;
	};
	category default { basic; };
};

key rndc_key {
    secret "1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

/* deliberately not _common/controls.conf.j2: non-ns<N> directory without the ns.ip template variable */
controls {
	inet 10.53.0.3 port @CONTROLPORT@ allow { any; } keys { rndc_key; };
};

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	listen-on { 10.53.0.3; };
	pid-file "named.pid";
	notify no;
	dnssec-validation no;
	allow-query { any; };
	allow-recursion { any; };
};

zone "." {
	type hint;
	file "root.db";
};
