{% if FEATURE_FIPS_DH == "1" %}
include "named-tls.conf";
{% endif %}

options {
	query-source address 10.53.0.4;
	notify-source 10.53.0.4;
	transfer-source 10.53.0.4;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
	minimal-responses yes;

{% if FEATURE_FIPS_DH == "1" %}
	tls-port @TLSPORT@;
{% endif %}

};

{% include "_common/controls.conf.j2" %}

{% if FEATURE_JSON_C == "1" %}
statistics-channels { inet 10.53.0.4 port @EXTRAPORT1@ allow { localhost; }; };
{% endif %}

zone "." {
	type hint;
	file "root.db";
};

zone "example5." {
	type forward;
	forward only;
	forwarders { 10.53.0.2; };
};

zone "grafted" {
	type forward;
	forward only;
	forwarders { 10.53.0.2; };
};

zone "malicious." {
	type primary;
	file "malicious.db";
};

zone "sibling.tld" {
	type primary;
	file "sibling.tld.db";
};
