// NS1

{% include "_common/controls.conf.j2" %}

acl rfc1918 { 10/8; 192.168/16; 172.16/12; };

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	allow-recursion { 10.53.0.1; };
	notify yes;
	dnssec-validation no;

	dns64 2001:bbbb::/96 {
		clients { any; };
		mapped { !rfc1918; any; };
		exclude { 2001:eeee::/32; 64:FF9B::/96; ::ffff:0000:0000/96; };
		suffix ::;
		recursive-only yes;
	};
	dns64 2001:aaaa::/64 {
		mapped { !rfc1918; any; };
	};
};

zone "." {
	type primary;
	file "root.db";
};

zone "example" {
	type primary;
	file "example.db";
};

zone "signed" {
	type primary;
	file "signed.db.signed";
};

zone "ipv4only.arpa" {
	type primary;
	file "ipv4only.arpa.db";
};
