// NS8

options {
	query-source address 10.53.0.8;
	notify-source 10.53.0.8;
	transfer-source 10.53.0.8;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.8; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "ns2" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns2.db";
};

zone "ns2-4" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns2-4.db";
};

zone "ns2-4-5" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns2-4-5.db";
};

zone "ns2-4-6" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns2-4-6.db";
};

zone "ns2-5-7" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns2-5-7.db";
};

zone "ns5" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns5.db";
};

zone "ns5-6-7" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns5-6-7.db";
};

zone "ns5-7" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns5-7.db";
};

zone "ns6" {
	type secondary;
	primaries source 10.53.0.8 { 10.53.0.2 port @PORT@; };
	file "ns6.db";
};
