options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	session-keyfile "session.key";
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
	querylog yes;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type primary;
	file "root.db";
};

zone "test.example.org" {
	type primary;
	file "example.db";
};

zone "l0" {
	type primary;
	file "db.l0";
};

zone "l1.l0" {
	type primary;
	file "db.l1.l0";
};

zone "l2.l1.l0" {
	type primary;
	file "does-not-exist";
};

zone "test1.example.net" {
	type primary;
	file "test1.example.net.db";
};

zone "test2.example.net" {
	type primary;
	file "test2.example.net.db";
};

zone "example.com" {
	type primary;
	file "example.com.db";
};
