{% set example_zone = example_zone | default(True) %}

{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
	notify no;
};

{% if example_zone %}
zone "example" {
	type primary;
	file "example.db.signed";
};
{% endif %}

zone "example.org" {
	type primary;
	file "example.org.db";
};

zone "undelegated" {
	type primary;
	file "undelegated.db.signed";
};

zone "unsigned" {
	type primary;
	file "unsigned.db";
};
