// NS9

options {
	query-source address 10.53.0.9;
	notify-source 10.53.0.9;
	transfer-source 10.53.0.9;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.9; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;

	max-recursion-queries 50;
	max-query-restarts 50;
	max-query-count 50;
};

{% include "_common/controls.conf.j2" %}

zone "." { type hint; file "hints.db"; };
