/*
 * Test stale-answer-client-timeout 0.
 */

{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	dnssec-validation no;
	recursion yes;
	stale-answer-enable yes;
	stale-cache-enable yes;
	stale-answer-ttl 3;
	stale-answer-client-timeout 0;
	stale-refresh-time 0;
	resolver-query-timeout 10000; # 10 seconds
	max-stale-ttl 3600;
	recursive-clients 10;
	prefetch 0;
};

zone "." {
	type hint;
	file "root.db";
};
