{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.6;
	notify-source 10.53.0.6;
	transfer-source 10.53.0.6;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.6; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
	notify yes;
	ixfr-from-differences primary;
	check-integrity no;
	tcp-idle-timeout 600;
	min-transfer-rate-in 10240 300;  # this is tested as seconds, when used with '-T transferinsecs' (i.e. convert the default '10240 5' back so that it doesn't interfere with other tests)
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "primary" {
	type primary;
	file "primary.db";
};

zone "secondary" {
	type secondary;
	notify no;
	primaries { 10.53.0.1; };
	file "sec.bk";
};

zone "edns-expire" {
	type secondary;
	primaries { 10.53.0.1; };
	file "edns-expire.bk";
};

zone "axfr-min-transfer-rate" {
	type secondary;
	min-transfer-rate-in 100000 5; # this is tested as seconds, when used with '-T transferinsecs' (i.e. 100000 bytes in 5 seconds)
	primaries { 10.53.0.1; };
	file "axfr-min-transfer-rate.bk";
};

zone "axfr-max-transfer-time" {
	type secondary;
	max-transfer-time-in 1; # this is tested as seconds, when used with '-T transferinsecs'
	primaries { 10.53.0.1; };
	file "axfr-max-transfer-time.bk";
};

zone "axfr-max-idle-time" {
	type secondary;
	max-transfer-idle-in 50;  # this is tested as seconds, when used with '-T transferinsecs'
	primaries { 10.53.0.1; };
	file "axfr-max-idle-time.bk";
};

zone "axfr-rndc-retransfer-force" {
	type secondary;
	primaries { 10.53.0.1; };
	file "axfr-rndc-retransfer-force.bk";
};

zone "xot-primary-try-next" {
	type secondary;
	primaries { 10.53.0.10 port @EXTRAPORT1@ tls ephemeral; 10.53.0.1; };
	file "xot-primary-try-next.bk";
};

zone "axfr-too-big" {
	type secondary;
	max-records 30;
	primaries { 10.53.0.1; };
	file "axfr-too-big.bk";
};

zone "ixfr-too-big" {
	type secondary;
	max-records 30;
	primaries { 10.53.0.1; };
	file "ixfr-too-big.bk";
};

zone "ixfr-too-many-diffs" {
	type secondary;
	request-ixfr-max-diffs 3;
	primaries { 10.53.0.1; };
	file "ixfr-too-many-diffs.bk";
};

zone "xfr-and-reconfig" {
	type secondary;
	primaries { 10.53.0.9; };
	file "xfr-and-reconfig.bk";
	request-ixfr no; # ans9 supports only axfr
};

# GL#5767
zone "ixfr-race" {
	type secondary;
	primaries { 10.53.0.11; };
	file "ixfr-race.bk";
};

# GL#6004
zone "private-dns-overrun" {
	type secondary;
	primaries { 10.53.0.9; };
	file "private-dns-overrun.bk";
	masterfile-format text; # force bug to be exercised
	request-ixfr no; # ans9 supports only axfr
};
