// NS6

options {
	port @PORT@;
	listen-on port @PORT@ { 10.53.0.6; };
	pid-file "named.pid";
	nxdomain-redirect unsigned;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
        type primary;
        file "root.db";
};

// A signed zone that ns5 has a delegation for.
zone "signed." {
        type primary;
        file "signed.db.signed";
};
