// NS2

{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	notify yes;
	notify-delay 0;
	allow-new-zones yes;
	dnssec-validation no;
};

zone "bits" {
	type primary;
	file "bits.db";
	allow-update { any; };
};

zone "retransfer" {
	type primary;
	file "retransfer.db";
	allow-update { any; };
	notify no;
};

zone "nsec3-loop" {
	type primary;
	file "nsec3-loop.db";
	notify no;
};

zone "nokeys" {
	type primary;
	file "nokeys.db";
	allow-update { any; };
};

zone "removedkeys-secondary" {
	type primary;
	file "removedkeys-secondary.db";
	allow-update { any; };
};
