{% include "_common/controls.conf.j2" %}

tls self-signed {
	cert-file "../self-signed-cert.pem";
	key-file "../self-signed-key.pem";
};

options {
	pid-file "named.pid";
	##
	# generic
	listen-on port @PORT@ { 10.53.0.1; };
	# test TLS
	listen-on port @EXTRAPORT1@ proxy encrypted tls self-signed { 10.53.0.1; };
	listen-on-v6 port @EXTRAPORT1@ proxy encrypted tls self-signed { fd92:7065:b8e:ffff::1; };
	##
	recursion no;
	notify explicit;
	statistics-file "named.stats";
	dnssec-validation no;
	tcp-initial-timeout 1200;
	allow-proxy { any; };
	allow-proxy-on { 10.53.0.1; fd92:7065:b8e:ffff::1; };
};


zone "example" {
	type primary;
	file "example.db";
	allow-transfer { any; };
};
