{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.7;
	notify-source 10.53.0.7;
	transfer-source 10.53.0.7;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.7; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
	qname-minimization off;

	stale-answer-enable yes;
	stale-cache-enable yes;
	max-stale-ttl 3600;

	stale-answer-client-timeout off;
	stale-refresh-time 30;

	max-cache-ttl 300;
	max-ncache-ttl 300;
};

zone "." {
	type hint;
	file "root.db";
};

// Authoritative zone: nonexist.target.stale -> NXDOMAIN
zone "target.stale" {
    type primary;
    file "target.stale.db";
};

// Forward source.stale queries to ans2
zone "source.stale" {
    type forward;
    forward only;
    forwarders { 10.53.0.2 port @PORT@; };
};
