options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; 10.53.0.4; 10.53.1.2; };
	listen-on-v6 { none; };
	recursion no;
	allow-query-cache { none; };
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

view internal {
        zone "." {
                type hint;
                file "../../_common/root.hint";
        };

        recursion yes;
        allow-query-cache { 10.53.0.3; 10.53.0.4; };
	allow-query { 10.53.0.4; };
};
