{% if zones is defined and zones %}
{% for zone in zones.values() %}
{% if zone.ns.name == ns.name %}
zone "@zone.name@" {
	type @zone.type@;
{% if zone.type == "static-stub" %}
        server-addresses { @zone.ns.ip@; };
{% else %}
	file "@zone.filepath@";
{% endif %}
};
{% endif %}
{% endfor %}
{% endif %}
