options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion yes;
	dnssec-validation no;
	notify yes;
};

{% include "_common/controls.conf.j2" %}

dnssec-policy "views" {
	keys {
		ksk key-directory lifetime unlimited algorithm @DEFAULT_ALGORITHM@;
		zsk key-directory lifetime unlimited algorithm @DEFAULT_ALGORITHM@;
	};
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "example" {
	type primary;
	file "example.db";
	allow-update { any; };
};

zone "inline" {
	type primary;
	file "external/inline.db";
	key-directory "external";
	dnssec-policy views;
	inline-signing yes;
};
