options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	notify yes;
	disable-empty-zone 127.IN-ADDR.ARPA;
	recursion yes;
	dnssec-validation no;

	// lame-and-expire-soon domain NS has a TTL of 2 seconds,
	// so let's use a lower value here.
	min-delegation-ttl 1;
};


{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "flushtest.example" {
	type forward;
	forwarders { 10.53.0.1; };
};

zone "expire-test" {
	type secondary;
	primaries { 10.53.0.1; };
};
