{% include "_common/controls.conf.j2" %}


options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; 10.53.0.4; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
};

template primary {
	type primary;
	file "$view-$name.db";
	initial-file "added.db";
};

view internal {
	match-clients { 10.53.0.2; };
	allow-new-zones no;
	recursion yes;

	response-policy { zone "policy"; };

	zone "." {
		type hint;
		file "../../_common/root.hint";
	};

	zone "policy" {
		type primary;
		file "normal.db";
	};
};

view external {
	match-clients { any; };
	allow-new-zones yes;

	zone "." {
		type hint;
		file "../../_common/root.hint";
	};
};

# This view is only here to test that configuration context is cleaned
# up correctly when using multiple named ACLs (regression test for RT #22739)
acl match { none; };
acl nobody { none; };
view extra {
	match-clients { match; };
	allow-new-zones yes;
	allow-transfer { nobody; };
	allow-query { nobody; };
	allow-recursion { nobody; };
};
