/*
 * Test serve-stale interaction with fetch-limits (dual-mode).
 */

{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	dnssec-validation no;
	recursion yes;
	/*
	 * stale-answer-enable is not strictly required because serving
	 * stale answers is enabled in the test via rndc.
	 */
	stale-answer-enable yes;
	stale-cache-enable yes;
	stale-answer-ttl 3;
	stale-refresh-time 4;
	resolver-query-timeout 10000; # 10 seconds
	fetches-per-zone 1 fail;
	fetches-per-server 1 fail;
	max-stale-ttl 3600;
	prefetch 0;
};

zone "." {
	type secondary;
	primaries { 10.53.0.1; };
	file "root.bk";
};
