options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-query { any; };
	recursion no;
	allow-new-zones yes;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

template primary {
	type primary;
	file "$view-$name.db";
	initial-file "added.db";
};

template bad {
	type secondary;
	primaries { 10.53.0.1; };
	max-refresh-time 0;
	max-retry-time 0;
	min-refresh-time 0;
	min-retry-time 0;
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "normal.example" {
	type primary;
	file "normal.db";
};

zone "finaldot.example." {
	type primary;
	file "normal.db";
};
