options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	dnssec-validation no;
	recursion no;
	notify yes;
};

logging {
	channel default_log {
	  buffered no;
	  file "named_unlimited" versions unlimited size 1000;
	  severity debug 100;
	  print-time yes;
	};
	category default { default_log; default_debug; };
	category lame-servers { null; };

	channel query_log {
	  file "query_log";
	  print-time yes;
	  buffered no;
	};
	category queries { query_log; };
};

{% include "_common/controls.conf.j2" %}
