// NS2

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	allow-notify { 10.53.0.3; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

{% for zone in tlds %}
zone "@zone@" {
	type primary;
	file "@zone@.db.signed";
};
{% endfor %}
