// NS2

options {
	query-source address 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { fd92:7065:b8e:ffff::2; };
	recursion no;
	dnssec-validation no;
	minimal-any no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "example" {
	type primary;
	file "example.db";
};

zone "example.tld" {
	type primary;
	file "example.tld.db";
	provide-zoneversion no;
};
