// NS3

include "../kasp.conf";

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	key-directory ".";
	dnssec-validation yes;
	notify-cfg CDS { notify yes; };
};

{% include "_common/controls.conf.j2" %}

zone "model2.multisigner." {
	type primary;
	allow-update { any; };
	file "model2.multisigner.db";
	dnssec-policy model2;
	inline-signing no;
};

zone "model2.update-any." {
	type primary;
	allow-update { any; };
	file "model2.update-any.db";
	dnssec-policy model2;
	inline-signing no;
};

zone "model2.bad-dsync." {
	type primary;
	allow-update { any; };
	file "model2.bad-dsync.db";
	dnssec-policy model2;
	inline-signing no;
};

zone "model2.secondary." {
	type secondary;
	primaries { 10.53.0.5; };
	file "model2.secondary.db";
	dnssec-policy model2;
	inline-signing yes;
	notify no;
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

include "trusted.conf";
