// NS2

dnssec-policy "nsec3" {
	nsec3param;
};

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	recursion no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

{% if "nsec3-xfr-inline.kasp" in zones %}
zone "nsec3-xfr-inline.kasp" {
	type primary;
	file "nsec3-xfr-inline.kasp.db";
	dnssec-policy "nsec3";
};
{% endif %}{# nsec3-xfr-inline.kasp #}

{% if "retransfer.kasp" in zones %}
zone "retransfer.kasp" {
	type primary;
	file "retransfer.kasp.db";
	notify explicit;
	also-notify { 10.53.0.3; };
	allow-transfer { any; };
};
{% endif %}{# nsec3-xfr-inline.kasp #}
