// NS5

{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.5;
	notify-source 10.53.0.5;
	transfer-source 10.53.0.5;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.5; };
	listen-on-v6 { none; };
	dnssec-validation no;
	recursion no;
	notify yes;
	notify-delay 0;
};

zone "bits" {
	type secondary;
	primaries { 10.53.0.2; };
	file "bits.bk";
};
