options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	ixfr-from-differences yes;
	check-integrity no;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

view compress {
	match-clients { 10.53.0.1/32; };
	zone "example" {
		type primary;
		file "example.db";
	};
};

view nocompress {
	match-clients { 10.53.0.2/32; };
	message-compression no;
	zone "example" {
		type primary;
		file "example.db";
	};
};
