options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	recursion no;
	dnssec-validation no;
	notify no;
};

{% include "_common/controls.conf.j2" %}

key-store "hsm" {
	directory ".";
	pkcs11-uri "pkcs11:token=softhsm2-enginepkcs11;pin-value=1234";
};

key-store "pin" {
	directory ".";
	pkcs11-uri "pkcs11:token=softhsm2-enginepkcs11;pin-source=pin";
};

key-store "disk" {
	directory "keys";
};
