// NS3

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;

	max-query-count 150;
};

{% include "_common/controls.conf.j2" %}

zone "final.tld0" {
	type primary;
	file "final.tld0.db";
};
