// NS2

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion yes;
	notify no;
	dnssec-validation yes;
};

{% include "_common/controls.conf.j2" %}

{% if FEATURE_LIBXML2 == "1" or FEATURE_JSON_C == "1" %}
statistics-channels {
	inet 10.53.0.2 port @EXTRAPORT1@ allow { any; };
};
{% endif %}

zone "." {
	type hint;
	file "root.hints";
};

zone "example.internal" {
	type primary;
	file "example.internal.db";
};

zone "example.internal2" {
	type primary;
	file "example.internal.db";
};

include "../ns1/trusted.conf";
