{% include "_common/controls.conf.j2" %}

tls self-signed {
	cert-file "../self-signed-cert.pem";
	key-file "../self-signed-key.pem";
};

options {
	pid-file "named.pid";
	##
	# generic test
	listen-on port @PORT@ { 10.53.0.3; };

	listen-on port @EXTRAPORT1@ proxy plain { 10.53.0.3; };
	listen-on port @TLSPORT@ proxy encrypted tls self-signed { 10.53.0.3; };
	listen-on port @EXTRAPORT2@ proxy plain tls self-signed { 10.53.0.3; };
	listen-on port @HTTPSPORT@ proxy encrypted tls self-signed http default { 10.53.0.3; };
	listen-on port @EXTRAPORT3@ proxy plain tls self-signed http default { 10.53.0.3; };
	listen-on port @HTTPPORT@ proxy plain tls none http default { 10.53.0.3; };

	listen-on-v6 port @EXTRAPORT1@ proxy plain { fd92:7065:b8e:ffff::3; };
	listen-on-v6 port @TLSPORT@ proxy encrypted tls self-signed { fd92:7065:b8e:ffff::3; };
	listen-on-v6 port @EXTRAPORT2@ proxy plain tls self-signed { fd92:7065:b8e:ffff::3; };
	listen-on-v6 port @HTTPSPORT@ proxy encrypted tls self-signed http default { fd92:7065:b8e:ffff::3; };
	listen-on-v6 port @EXTRAPORT3@ proxy plain tls self-signed http default { fd92:7065:b8e:ffff::3; };
	listen-on-v6 port @HTTPPORT@ proxy plain tls none http default { fd92:7065:b8e:ffff::3; };

	recursion no;
	notify explicit;
	statistics-file "named.stats";
	dnssec-validation no;
	tcp-initial-timeout 1200;
};


zone "example" {
	type primary;
	file "example.db";
	allow-query { any; };
};
