{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	listen-on { 10.53.0.1; };
	pid-file "named.pid";
	notify no;
	dnssec-validation no;
	allow-query { any; };
	recursion yes;
	allow-recursion { any; };
};

# Delegate .test domain to 10.53.0.2
zone "." {
    type primary;
    file "root.db";
    allow-transfer { none; };
};
