{% set allownewzones = allownewzones | default(True) %}

{% include "_common/controls.conf.j2" %}

options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	allow-transfer { any; };
	allow-query { any; };
{% if allownewzones %}
	allow-new-zones yes;
{% else %}
	allow-new-zones no;
{% endif %}
	recursion no;
	dnssec-validation no;
};

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "inlinesec.example" {
	type primary;
	file "inlinesec.db";
};

zone "." {
	type redirect;
	file "redirect.db";
};
