// NS7

options {
	query-source address 10.53.0.7;
	notify-source 10.53.0.7;
	transfer-source 10.53.0.7;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.7; };
	listen-on-v6 { none; };
	recursion yes;
	notify no;
	dnssec-validation auto;
};

{% include "_common/controls.conf.j2" %}

view view1 {
	zone "." {
		type hint;
		file "../../_common/root.hint";
	};
};

view view2 {
	zone "." {
		type hint;
		file "../../_common/root.hint";
	};
};
