{% include "_common/controls.conf.j2" %}

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion no;
	notify yes;
	ixfr-from-differences yes;
	check-integrity no;
	dnssec-validation no;
};

key one {
	algorithm @DEFAULT_HMAC@;
	secret "1234abcd8765";
};

key two {
	algorithm @DEFAULT_HMAC@;
	secret "1234abcd8765";
};

acl rejectkeys {
	!key one; !key two; any;
};

acl rejectaddrs {
	!10.53.0.1; !10.53.0.2; any;
};

acl check1 { !key one; 10.53.0.1; };

acl check2 { !key two; 10.53.0.2; };

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "example" {
	type primary;
	file "example.db";
};

zone "tsigzone" {
	type primary;
	file "tsigzone.db";
	allow-transfer { !rejectkeys; !rejectaddrs; !check1; !check2; any; };
};
