plugin query "@TOP_BUILDDIR@/filter-aaaa.@DYLIB@" {
	/*
	 * While this matches the defaults, it is not a good configuration
	 * to have in named.conf as the two options contradict each other
	 * indicating a error on behalf of the operator.
	 *
	 * The default is to have filter-aaaa-on-v4 off, but if it is turned
	 * on then it applies to all IPv4 queries.  This results in
	 * contradictory defaults.
	 */
	filter-aaaa-on-v4 no;
	filter-aaaa { any; };
};
