acl rfc1918 { 10/8; 192.168/16; 172.16/12; };

options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	allow-recursion { 10.53.0.3; };
	notify yes;
	dnssec-validation no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type primary;
	file "root.db";
};

zone "example" {
	type primary;
	file "example.db";
};

zone "signed" {
	type primary;
	file "signed.db.signed";
};

zone "nsec3" {
	type primary;
	file "nsec3.db.signed";
};

zone "redirect" {
	type primary;
	file "redirect.db";
};
