options {
	query-source address 10.53.0.3;
	notify-source 10.53.0.3;
	transfer-source 10.53.0.3;
	port @PORT@;
	directory ".";
	pid-file "named.pid";
	listen-on { 10.53.0.3; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
	notify yes;
	fetches-per-zone 40;
};


server 10.53.0.4 {
	edns no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "root.hint";
};
