// validating resolver

options {
	query-source address 10.53.0.2;
	notify-source 10.53.0.2;
	transfer-source 10.53.0.2;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.2; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation yes;
	minimal-responses no;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};


{% if PARENT_DNSKEY is defined and PARENT_DNSKEY|length %}
zone "f045.test" {
        type static-stub;
        server-addresses { 10.53.0.1; };
};

trust-anchors {
        f045.test. static-key 257 3 13 "@PARENT_DNSKEY@";
};
{% else %}
zone "f043.test" {
	type static-stub;
	server-addresses { 10.53.0.1; };
};

trust-anchors {
	f043.test. static-key 257 3 13 "@TESTZONE_DNSKEY@";
};
{% endif %}
