options {
	listen-on port @PORT@ { 10.53.0.4; };
	transfer-source 10.53.0.4;
	pid-file "named.pid";
	recursion no;
};

key xfrkey {
	algorithm @DEFAULT_HMAC@;
	secret "9999abcd8765";
};

key notifykey4 {
	algorithm @DEFAULT_HMAC@;
	secret "4444abcd8765";
};

zone "test" {
	/*
	 * Notify comes from a different IP address than the primary listening
	 * address, and with a different key.
	 */
	allow-notify { key notifykey4; };
	type secondary;
	primaries { 10.53.0.1 port @PORT@ key xfrkey; };
};

{% include "_common/controls.conf.j2" %}
