// NS7

options {
	query-source address 10.53.0.7;
	notify-source 10.53.0.7;
	transfer-source 10.53.0.7;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.7; };
	listen-on-v6 { none; };
	recursion yes;
	qname-minimization relaxed;
	querylog yes;
	resolver-query-timeout 30000; # 30 seconds
	dnssec-validation no;
	disable-empty-zone 10.in-addr.arpa;
};

{% include "_common/controls.conf.j2" %}

zone "." {
	type hint;
	file "../../_common/root.hint";
};

zone "fwd." IN {
	type forward;
	forwarders {
		10.53.0.2;
	};
	forward only;
};
