/* deliberately not _common/controls.conf.j2: setup.sh appends a runtime-generated multi-key channel on EXTRAPORT7 */
options {
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.4; };
	listen-on-v6 { none; };
	recursion yes;
	dnssec-validation no;
};


view normal {
	match-clients { any; };

	zone example {
		type primary;
		file "example.db";
		allow-update { any; };
	};
};

view "view with a space" {
	match-clients { none; };
	zone example {
		in-view normal;
	};
};
