{% set policy = policy | default("default") %}

include "kasp.conf";
include "named.common.conf";

zone "going-straight-to-none.kasp" {
	type primary;
	file "going-straight-to-none.kasp.db";
	dnssec-policy @policy@;
};

zone "going-straight-to-none-dynamic.kasp" {
	type primary;
	file "going-straight-to-none-dynamic.kasp.db.signed";
	inline-signing no;
	dnssec-policy @policy@;
	allow-update { any; };
};
